/*
 * Libart Sample
 * ------------------------------------------------------------------------
 * Licensed under the BSD license, see LICENSE for details.
 *
 * Copyright (c) 2005 James Forshaw <tyranid@gmail.com>
 * Copyright (c) 2005 Frank Buss <fb@frank-buss.de> (aka Shine)
 *
 * This program reads data from the serial port at 9600 baud and
 * writes it back to it, all from user mode thread with stdout.
 * TODO: stdin doesn't work.
 *
 * $Id$
 */

#include <pspkernel.h>
#include <pspge.h>
#include <pspdisplay.h>
#include <pspsdk.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <libart_lgpl/art_misc.h>
#include <libart_lgpl/art_vpath.h>
#include <libart_lgpl/art_svp.h>
#include <libart_lgpl/art_svp_vpath.h>
#include <libart_lgpl/art_gray_svp.h>
#include <libart_lgpl/art_rgb_svp.h>
#include <libart_lgpl/art_svp_vpath_stroke.h>
#include <libart_lgpl/art_svp_ops.h>
#include <libart_lgpl/art_affine.h>
#include <libart_lgpl/art_rgb_affine.h>
#include <libart_lgpl/art_rgb_bitmap_affine.h>
#include <libart_lgpl/art_rgb_rgba_affine.h>
#include <libart_lgpl/art_alphagamma.h>
#include <libart_lgpl/art_svp_point.h>
#include <libart_lgpl/art_vpath_dash.h>
#include <libart_lgpl/art_render.h>
#include <libart_lgpl/art_render_gradient.h>
#include <libart_lgpl/art_render_svp.h>

/* Define the module info section */
PSP_MODULE_INFO("LIBARTSAMPLE", 0x1000, 1, 1);

/* Define the main thread's attribute value (optional) */
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER | THREAD_ATTR_VFPU);

u32* g_vram_base;

/* Exit callback */
int exit_callback(void)
{
	sceKernelExitGame();
	return 0;
}

/* Callback thread */
int CallbackThread(SceSize args, void *argp)
{
	int cbid;

	cbid = sceKernelCreateCallback("Exit Callback", (void *) exit_callback, NULL);
	sceKernelRegisterExitCallback(cbid);

	sceKernelSleepThreadCB();

	return 0;
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void)
{
	int thid = 0;

	thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, 0, 0);
	if(thid >= 0)
	{
		sceKernelStartThread(thid, 0, 0);
	}

	return thid;
}

static ArtVpath *
randstar (int n)
{
	ArtVpath *vec;
	int i;
	double r, th;
	
	vec = art_new (ArtVpath, n + 2);
	for (i = 0; i < n; i++)
	{
		vec[i].code = i ? ART_LINETO : ART_MOVETO;
		r = rand () * (250.0 / RAND_MAX);
		#if 0
		r = r + 0.9 * (250 - r);
		#endif
		th = i * 2 * M_PI / n;
		vec[i].x = 250 + r * cos (th);
		vec[i].y = 250 - r * sin (th);
	}
	vec[i].code = ART_LINETO;
	vec[i].x = vec[0].x;
	vec[i].y = vec[0].y;
	i++;
	vec[i].code = ART_END;
	vec[i].x = 0;
	vec[i].y = 0;
	return vec;
}

void testLibart()
{
	//art_u8* buf = calloc(512 * 512 * 3, 1);
	ArtGradientLinear gradient;
	ArtGradientStop stops[3] = {
		{ 0.0, { 0x7fff, 0x0000, 0x0000, 0x7fff }},
		{ 0.5, { 0x0000, 0x0000, 0x0000, 0x1000 }},
		{ 1.0, { 0x0000, 0x7fff, 0x0000, 0x7fff }}
	};
	ArtVpath *vpath;
	ArtSVP *svp;
	ArtRender *render;
	
	gradient.a = 0.003;
	gradient.b = -0.0015;
	gradient.c = 0.1;
	gradient.spread = ART_GRADIENT_PAD;
	gradient.n_stops = sizeof(stops) / sizeof(stops[0]);
	gradient.stops = stops;
	
	vpath = randstar (50);
	svp = art_svp_from_vpath (vpath);
	
	render = art_render_new (0, 0, 512, 512, g_vram_base, 512 * 4, 4, 8, ART_ALPHA_NONE,
			   NULL);
	art_render_svp (render, svp);
	art_render_gradient_linear (render, &gradient, ART_FILTER_NEAREST);
	art_render_invoke (render);

/*
	int x, y;
	for (y = 0; y < 272; y++) {
		for (x = 0; x < 480; x++) {
			int r = buf[3 * (x + y * 512)];
			int g = buf[3 * (x + y * 512) + 1];
			int b = buf[3 * (x + y * 512) + 2];
			g_vram_base[x + y * 512] = r | (g << 8) | (b << 16);
		}
	}
	free(buf);*/
}

int main(int argc, char** argv)
{
	SetupCallbacks();
	
	g_vram_base = (void *) (0x40000000 | (u32) sceGeEdramGetAddr());
	sceDisplaySetMode(0, 512, 272);
	sceDisplaySetFrameBuf((void *) g_vram_base, 512, PSP_DISPLAY_PIXEL_FORMAT_8888, 1);

	while(1) {
		testLibart();
		sceDisplayWaitVblankStart();
	}

	return 0;
}
